"""Spanishdict cli

  Usage:
      python spanishdict <subcommand>
  
  Subcommands:
      translate [words]
      conjugate FLAGS [mood] [words]
          Flags:
              -p, --participle get the particple of verbs
          Mood:
              indicative(default)
              subjunctive
              imperative
              continuous
              perfect
              perfect-subjunctive
"""
